<?php

namespace Drupal\entity_workflow_content\Routing;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Routing\EnhancerInterface;
use Drupal\Core\Routing\RouteObjectInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\workflows\Entity\Workflow;
use Symfony\Component\HttpFoundation\Request;

/**
 * Swaps the controller for routes that need to check for an active workspace.
 */
class RouteEnhancer implements EnhancerInterface {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The workflow type plugin manager.
   *
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected $workflowTypeManager;

  /**
   * Constructs a RouteEnhancer object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Component\Plugin\PluginManagerInterface $workflow_type_manager
   *   The workflow type plugin manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, PluginManagerInterface $workflow_type_manager) {
    // This service is injected because it's also used by other route enhancers.
    $this->entityTypeManager = $entity_type_manager;
    $this->workflowTypeManager = $workflow_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function enhance(array $defaults, Request $request) {
    /** @var \Symfony\Component\Routing\Route $route */
    $route = $defaults[RouteObjectInterface::ROUTE_OBJECT];

    // Return early if we don't need to require an active workspace.
    if (!$route->hasOption('_entity_workflow_content.require_workspace')) {
      return $defaults;
    }

    $entity_type_id = $route->getOption('_entity_workflow_content.entity_type_id');

    // Try to get the bundle name on entity add routes.
    if (($bundle_entity_type = $this->entityTypeManager->getDefinition($entity_type_id)->getBundleEntityType()) && isset($defaults[$bundle_entity_type])) {
      $bundle = $defaults['_raw_variables']->get($bundle_entity_type);
    }
    // Try to get the bundle name on entity edit, Layout Builder and
    // Entityqueue routes.
    if (isset($defaults[$entity_type_id]) && $defaults[$entity_type_id] instanceof EntityInterface) {
      $bundle = $defaults[$entity_type_id]->bundle();
    }

    // Entire entity types or individual bundles may have been excluded from
    // the workspace requirement.
    $workflow = Workflow::load('content');
    $workflow_config = $workflow->getTypePlugin()->getConfiguration();
    if (!isset($bundle) && array_key_exists($entity_type_id, $workflow_config['exclude_from_workspace_requirement'])) {
      return $defaults;
    }
    elseif (
      isset($bundle) &&
      array_key_exists($entity_type_id, $workflow_config['exclude_from_workspace_requirement']) &&
      (
        empty($workflow_config['exclude_from_workspace_requirement'][$entity_type_id]) ||
        in_array($bundle, $workflow_config['exclude_from_workspace_requirement'][$entity_type_id], TRUE)
      )
    ) {
      return $defaults;
    }

    if (!isset($bundle)) {
      $workflow_info = $this->getWorkflowInfo()->getWorkflowsInfoForEntityType($entity_type_id);
    }
    else {
      $workflow_info = $this->getWorkflowInfo()->getWorkflowsInfoForEntityTypeAndBundle($entity_type_id, $bundle);
    }

    // If the 'content' workflow is enabled for this entity type and bundle,
    // swap the controller of the route to our workspace switcher controller.
    // When the workspace switcher form is submitted, either by selecting an
    // existing workspace or adding a new one, Drupal will redirect to the
    // same page, but at that point there will be an active workspace so the
    // controller won't be swapped anymore.
    if (isset($workflow_info['content'])) {
      if (!$this->getWorkspaceManager()->hasActiveWorkspace()) {
        $defaults['_controller'] = '\Drupal\entity_workflow_content\Controller\WorkspaceSwitcherController::switcher';
      }
      elseif (($workflow = Workflow::load('workspace')) && $this->getWorkspaceManager()->hasActiveWorkspace()) {
        // When we also have a workspace workflow, ensure that content can not
        // be edited in locked or closed states.
        $workspace = $this->getWorkspaceManager()->getActiveWorkspace();
        if ($state_id = entity_workflow_get_entity_state($workspace, 'workspace')) {
          /** @var \Drupal\entity_workflow_workspace\WorkspaceState $state */
          $state = $workflow->getTypePlugin()->getState($state_id);

          if ($state->getStatus() === WSE_STATUS_CLOSED) {
            $defaults['_controller'] = '\Drupal\entity_workflow_content\Controller\WorkspaceSwitcherController::closed';
          }
          elseif ($state->isLocked()) {
            $defaults['_controller'] = '\Drupal\entity_workflow_content\Controller\WorkspaceSwitcherController::locked';
          }
        }
      }
    }

    return $defaults;
  }

  /**
   * @return \Drupal\workspaces\WorkspaceManagerInterface
   *   The workspace manager.
   */
  protected function getWorkspaceManager() {
    // We're not injecting this service so we don't initialize it unless it's
    // actually needed.
    return \Drupal::service('workspaces.manager');
  }

  /**
   * @return \Drupal\entity_workflow\EntityWorkflowInfo
   *   The workflow information service.
   */
  protected function getWorkflowInfo() {
    // We're not injecting this service so we don't initialize it unless it's
    // actually needed.
    return \Drupal::service('entity_workflow.info');
  }

}
