<?php

namespace Drupal\entity_workflow_content\Routing;

use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\RouteSubscriberBase;
use Drupal\Core\Routing\RoutingEvents;
use Drupal\entity_workflow\EntityWorkflowInfo;
use Drupal\trash\TrashManagerInterface;
use Symfony\Component\Routing\RouteCollection;

/**
 * Subscriber for Entity Workflow Content routes.
 */
class RouteSubscriber extends RouteSubscriberBase {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity workflow information service.
   *
   * @var \Drupal\entity_workflow\EntityWorkflowInfo
   */
  protected $entityWorkflowInfo;

  /**
   * Constructs a RouteSubscriber object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\entity_workflow\EntityWorkflowInfo $entity_workflow_info
   *   The entity workflow info service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, EntityWorkflowInfo $entity_workflow_info) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityWorkflowInfo = $entity_workflow_info;
  }

  /**
   * {@inheritdoc}
   */
  protected function alterRoutes(RouteCollection $collection) {
    foreach ($this->entityTypeManager->getDefinitions() as $entity_type_id => $entity_type) {
      $workflows_info = $this->entityWorkflowInfo->getWorkflowsInfoForEntityType($entity_type_id);
      if (isset($workflows_info['content'])) {
        // Ensure that all form operations (e.g. Add/Edit, Layout Builder,
        // Entityqueue, etc.) require an active workspace for supported entity
        // types.
        $this->handleEntityForms($collection, $entity_type);
      }
    }
  }

  /**
   * Enhances various entity forms in the Live workspace.
   *
   * @param \Symfony\Component\Routing\RouteCollection $collection
   *   The route collection.
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type.
   */
  protected function handleEntityForms(RouteCollection $collection, EntityTypeInterface $entity_type) {
    // Handle entity forms.
    foreach ($collection as $route) {
      $has_content_workflow = FALSE;

      // Handle 'add' forms.
      if ($entity_create_requirement = $route->getRequirement('_entity_create_access')) {
        [$entity_type_id] = explode(':', $entity_create_requirement);
        if ($entity_type_id === $entity_type->id()) {
          $has_content_workflow = TRUE;
        }
      }

      // Handle 'edit/delete' forms.
      if ($entity_form = $route->getDefault('_entity_form')) {
        [$entity_type_id, $operation] = explode('.', $entity_form, 2);
        if ($entity_type_id === $entity_type->id()) {
          // Don't override the delete controller for entity types that can be
          // managed in a workspace, but are not enabled in Trash.
          if ($operation === 'delete'
            && ($trash_manager = $this->getTrashManager())
            && !$trash_manager->isEntityTypeEnabled($entity_type)
          ) {
            $has_content_workflow = FALSE;
          }
          else {
            $has_content_workflow = TRUE;
          }
        }
      }

      if ($has_content_workflow) {
        $route->setOption('_entity_workflow_content.require_workspace', TRUE);
        $route->setOption('_entity_workflow_content.entity_type_id', $entity_type->id());
      }
    }

    // Handle Layout Builder forms.
    $suffixes = ['view', 'discard_changes', 'revert'];
    foreach ($suffixes as $suffix) {
      if ($route = $collection->get("layout_builder.overrides.{$entity_type->id()}.$suffix")) {
        $route->setOption('_entity_workflow_content.require_workspace', TRUE);
        $route->setOption('_entity_workflow_content.entity_type_id', $entity_type->id());
      }
    }

    // Handle Entityqueue forms.
    if ($route = $collection->get("entity.{$entity_type->id()}.entityqueue")) {
      $route->setOption('_entity_workflow_content.require_workspace', TRUE);
      $route->setOption('_entity_workflow_content.entity_type_id', $entity_type->id());
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events = parent::getSubscribedEvents();
    // Ensure that we run after Entityqueue's route subscriber so that we can
    // handle those routes as well.
    $events[RoutingEvents::ALTER] = ['onAlterRoutes', -215];
    return $events;
  }

  /**
   * @return \Drupal\trash\TrashManagerInterface|null
   *   The trash manager.
   */
  protected function getTrashManager(): ?TrashManagerInterface {
    // We're not injecting this service, so we don't initialize it unless it's
    // actually needed.
    return \Drupal::hasService('trash.manager')
      ? \Drupal::service('trash.manager')
      : NULL;
  }

}
