<?php

namespace Drupal\entity_workflow_workspace;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\workflows\Entity\Workflow;
use Drupal\workspaces\WorkspaceInterface;
use Drupal\wse\WseWorkspaceListBuilder;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Overrides the workspace list builder to customize the workspace switcher.
 */
class EntityWorkflowWorkspaceListBuilder extends WseWorkspaceListBuilder {

  /**
   * The state transition validation service.
   *
   * @var \Drupal\entity_workflow\StateTransitionValidatorInterface
   */
  protected $stateTransitionValidator;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    $instance = parent::createInstance($container, $entity_type);
    $instance->stateTransitionValidator = $container->get('entity_workflow.state_transition_validation');
    $instance->currentUser = $container->get('current_user');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    // Update the status options with the states from the workspace workflow.
    $workflow = Workflow::load('workspace');
    $states = $workflow->getTypePlugin()->getConfiguration()['states'];
    $status_options = $form['status']['#options'];

    $options = [];
    foreach ($states as $state_id => $state) {
      $status = $state['status'] ?? WSE_STATUS_OPEN;
      $options[(string) $status_options[$status]][$state_id] = $state['label'];
    }
    $form['status']['#title'] = $this->t('State');
    $form['status']['#options'] = $options;

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function filterByStatus(&$entities) {
    // The parent method filters by 'open' workspaces by default. We keep that
    // behavior unless the user selected a specific workflow state to filter by.
    if ($status = $this->requestStack->getCurrentRequest()->query->get('status')) {
      // Filter workspaces to display only those that have a certain state.
      $entities = array_filter($entities, function ($workspace) use ($status) {
        $state = entity_workflow_get_entity_state($workspace, 'workspace');
        return !$state || $state === $status;
      });
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header = parent::buildHeader();

    $header['state'] = $this->t('State');

    $operations = $header['operations'];
    unset($header['operations']);
    $header['operations'] = $operations;

    return $header;
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    $row = parent::buildRow($entity);

    $row['data']['state'] = entity_workflow_get_field($entity, 'workspace')->getStateLabel();

    $operations = $row['data']['operations'];
    unset($row['data']['operations']);
    $row['data']['operations'] = $operations;

    return $row;
  }

  /**
   * {@inheritdoc}
   */
  protected function offCanvasRender(array &$build) {
    parent::offCanvasRender($build);

    // Swap out the main action from the toolbar workspace switcher.
    if (isset($build['active_workspace']['actions']['publish'])) {
      $build['active_workspace']['actions']['publish']['#access'] = FALSE;
    }

    if ($active_workspace = $this->workspaceManager->getActiveWorkspace()) {
      // Add an action button for each available transition.
      foreach ($this->getWorkspaceWorkflowTransitions($active_workspace) as $transition_info) {
        $build['active_workspace']['actions'][$transition_info['id']] = [
          '#type' => 'link',
          '#title' => $transition_info['label'],
          '#url' => $transition_info['url'],
          '#attributes' => [
            'class' => ['button', 'active-workspace__button'],
          ],
        ];
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity) {
    $operations = parent::getDefaultOperations($entity);

    // Publishing and reverting a workspace are now done via workflow
    // transitions.
    unset($operations['publish'], $operations['revert']);

    // Add an operation for each available transition.
    foreach ($this->getWorkspaceWorkflowTransitions($entity) as $transition_info) {
      $operations[$transition_info['id']] = [
        'title' => $transition_info['label'],
        'url' => $transition_info['url'],
        'weight' => 20,
      ];
    }

    return $operations;
  }

  /**
   * Returns the available workflow transitions for a workspace.
   *
   * @param \Drupal\workspaces\WorkspaceInterface $workspace
   *   A workspace entity.
   *
   * @return array
   *   An array with the following key/value pairs:
   *   - id: The transition ID.
   *   - label: The transition label.
   *   - url: The transition URL.
   */
  protected function getWorkspaceWorkflowTransitions(WorkspaceInterface $workspace) {
    $transitions = [];

    $workflow = Workflow::load('workspace');
    $available_transitions = $this->stateTransitionValidator->getValidTransitions($workspace, $workflow, $this->currentUser);
    foreach ($available_transitions as $transition) {
      $transitions[] = [
        'id' => $transition->id(),
        'label' => $transition->label(),
        'url' => Url::fromRoute("entity.workspace.workflow_transition", [
          'workspace' => $workspace->id(),
          'workflow' => $workflow->id(),
          'transition_id' => $transition->id(),
        ],
        [
          'query' => $this->getRedirectDestination()->getAsArray(),
        ]),
      ];
    }

    return $transitions;
  }

}
