<?php

namespace Drupal\entity_workflow_workspace\Plugin\WorkflowType;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\entity_workflow\WorkflowType\EntityWorkflowTypeBase;
use Drupal\entity_workflow_workspace\WorkspaceState;
use Drupal\workflows\TransitionInterface;
use Drupal\workflows\WorkflowInterface;

/**
 * @WorkflowType(
 *   id = "entity_workflow_workspace",
 *   label = @Translation("Workspace"),
 *   required_states = {
 *     "draft",
 *     "review",
 *     "approved",
 *     "published",
 *   },
 *   forms = {
 *     "configure" = "\Drupal\entity_workflow\Form\EntityWorkflowConfigureForm",
 *     "state" = "\Drupal\entity_workflow_workspace\Form\WorkspaceStateForm",
 *     "transition" = "\Drupal\entity_workflow\Form\EntityWorkflowTransitionForm",
 *   },
 * )
 */
class EntityWorkflowWorkspace extends EntityWorkflowTypeBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'entity_types' => [
        'workspace' => [],
      ],
      'default_state' => 'draft',
      'default_transition' => 'new_draft',
      'states' => [
        'draft' => [
          'label' => t('Draft'),
          'weight' => 0,
          'locked' => FALSE,
          'status' => WSE_STATUS_OPEN,
        ],
        'review' => [
          'label' => t('Ready for Review'),
          'weight' => 30,
          'locked' => FALSE,
          'status' => WSE_STATUS_OPEN,
        ],
        'approved' => [
          'label' => t('Approved'),
          'weight' => 50,
          'locked' => TRUE,
          'status' => WSE_STATUS_OPEN,
        ],
        'published' => [
          'label' => t('Published'),
          'weight' => 100,
          'locked' => TRUE,
          'status' => WSE_STATUS_CLOSED,
        ],
      ],
      'transitions' => [
        'new_draft' => [
          'label' => t('New draft'),
          'from' => [],
          'to' => 'draft',
          'weight' => 0,
        ],
        'submit_for_review' => [
          'label' => t('Submit for Review'),
          'from' => ['draft'],
          'to' => 'review',
          'transition_log' => TRUE,
          'action_link' => TRUE,
          'weight' => 10,
        ],
        'approve' => [
          'label' => t('Approve'),
          'from' => ['review'],
          'to' => 'approved',
          'transition_log' => TRUE,
          'action_link' => TRUE,
          'weight' => 100,
        ],
        'back_to_draft' => [
          'label' => t('Back to Draft'),
          'from' => ['review', 'approved'],
          'to' => 'draft',
          'transition_log' => TRUE,
          'action_link' => TRUE,
          'weight' => 20,
        ],
        'publish' => [
          'label' => t('Publish'),
          'from' => ['approved'],
          'to' => 'published',
          'transition_log' => TRUE,
          'action_link' => TRUE,
          'weight' => 100,
        ],
        'unpublish' => [
          'label' => t('Unpublish'),
          'from' => ['published'],
          'to' => 'draft',
          'transition_log' => TRUE,
          'action_link' => TRUE,
          'weight' => 110,
        ],
      ],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function getPermissions() {
    $perms = [];

    // Add generic permission to transition to any state.
    $perms['use all workspace transitions'] = [
      'title' => $this->t('%workflow workflow: Use all transitions', [
        '%workflow' => $this->label(),
      ]),
      'description' => $this->t('Allow the user to transition from any state to any state, as defined in the %workflow workflow.', [
        '%workflow' => $this->label(),
      ]),
    ];

    foreach ($this->getTransitions() as $transition) {
      $perms['use workspace transition ' . $transition->id()] = [
        'title' => $this->t('%workflow workflow: Use %transition transition', [
          '%workflow' => $this->label(),
          '%transition' => $transition->label(),
        ]),
        'description' => $this->t('Allow the user to use the %transition transition, as defined in the %workflow workflow.', [
          '%workflow' => $this->label(),
          '%transition' => $transition->label(),
        ]),
      ];
    }

    $perms['publish workspaces always'] = [
      'title' => $this->t('Can always publish workspaces.'),
      'description' => $this->t('Allows user to publish a workspace, ignoring its workflow.'),
    ];

    return $perms;
  }

  public function onPreTransition(TransitionInterface $transition, WorkflowInterface $workflow, ContentEntityInterface $entity, $field_name, array $context = []) {
    parent::onPreTransition($transition, $workflow, $entity, $field_name, $context);

    // Determine the new workspace status (e.g. 'open' or 'closed') based on the
    // transition being applied.
    /** @var \Drupal\entity_workflow_workspace\WorkspaceState $to_state */
    $to_state = $transition->to();
    $entity->set('status', $to_state->getStatus());
  }

  /**
   * {@inheritdoc}
   */
  public function getState($state_id) {
    if (!isset($this->configuration['states'][$state_id])) {
      throw new \InvalidArgumentException("The state '$state_id' does not exist in workflow.");
    }
    return new WorkspaceState(
      $this,
      $state_id,
      $this->configuration['states'][$state_id]
    );
  }

}
