<?php

namespace Drupal\entity_workflow;

use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemList;
use Drupal\Core\Form\FormStateInterface;
use Drupal\workflows\Entity\Workflow;

/**
 * Defines an item list class for the 'entity_workflow_state' field type.
 */
class EntityWorkflowStateFieldItemList extends FieldItemList {

  /**
   * {@inheritdoc}
   */
  public function preSave() {
    parent::preSave();

    /** @var \Drupal\Core\Entity\RevisionableInterface $entity */
    $entity = $this->getEntity();

    // Ensure that workflow states are removed when saving an entity in the
    // default (live) workspace.
    if ($entity->isDefaultRevision() && $entity->getEntityTypeId() !== 'workspace' && !$this->isEmpty()) {
      $this->list[0]->set('value',  NULL);
      return;
    }

    // Skip applying any default transition if this entity is saved in the
    // default (Live) workspace.
    // @todo Except for workspace entities. Should be controlled by the workflow
    //   plugin?
    if (!\Drupal::service('workspaces.manager')->hasActiveWorkspace()) {
      return;
    }

    // Skip workflow fields that already have a state set.
    if (!$this->isEmpty()) {
      return;
    }

    /** @var \Drupal\entity_workflow\Plugin\Field\FieldType\EntityWorkflowStateItem $item */
    $item = $this->appendItem();
    /** @var \Drupal\entity_workflow\WorkflowType\EntityWorkflowTypeInterface $workflow */
    $workflow = $item->getWorkflow()->getTypePlugin();

    // If there is no default transition to write, or the current state on the
    // field is not the default state.
    if (!$workflow->getDefaultTransition() || ($item->value && ($workflow->getDefaultState() != $item->value))) {
      return;
    }

    // When adding an entity to a workspace, we may have a default transition
    // requested. If so, record it so that the history is maintained.
    $transition = $workflow->getTransition($workflow->getDefaultTransition());
    $context['log'] = $workflow->getDefaultTransitionLog() ?? '';
    $context['default_transition'] = TRUE;
    $item->applyTransition($transition, $context);

    // Execute the field item pre-save method _again_ after applying a default
    // transition in order to ensure that transition events are invoked.
    parent::preSave();
  }

  /**
   * {@inheritdoc}
   */
  public static function processDefaultValue($default_value, FieldableEntityInterface $entity, FieldDefinitionInterface $definition) {
    // The default value of a workflow state field can only be the initial state
    // of the workflow.
    $storage_definition = $definition->getFieldStorageDefinition();
    if ($callback = $storage_definition->getSetting('workflow_callback')) {
      $workflow_id = call_user_func($callback, $entity);
    }
    else {
      $workflow_id = $storage_definition->getSetting('workflow');
    }

    /** @var \Drupal\workflows\WorkflowInterface $workflow */
    $workflow = Workflow::load($workflow_id);
    return [$workflow->getTypePlugin()->getInitialState($entity)->id()];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultValuesForm(array &$form, FormStateInterface $form_state) {
    $workflow = $this->list[0]->getWorkflow();
    return [
      '#markup' => $workflow->getTypePlugin()->getState($this->list[0]->value)->label(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function defaultValueWidget(FormStateInterface $form_state) {
    return NULL;
  }

}
