<?php

namespace Drupal\entity_workflow\EventSubscriber;

use Drupal\Core\Entity\EntityDefinitionUpdateManagerInterface;
use Drupal\Core\Entity\EntityLastInstalledSchemaRepositoryInterface;
use Drupal\Core\Entity\EntityTypeEventSubscriberTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeListenerInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\entity_workflow\EntityWorkflowInfo;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Defines a class for listening to entity schema changes.
 */
class EntitySchemaSubscriber implements EntityTypeListenerInterface, EventSubscriberInterface {

  use EntityTypeEventSubscriberTrait;
  use StringTranslationTrait;

  /**
   * The definition update manager.
   *
   * @var \Drupal\Core\Entity\EntityDefinitionUpdateManagerInterface
   */
  protected $entityDefinitionUpdateManager;

  /**
   * The last installed schema definitions.
   *
   * @var \Drupal\Core\Entity\EntityLastInstalledSchemaRepositoryInterface
   */
  protected $entityLastInstalledSchemaRepository;

  /**
   * The entity workflow information service.
   *
   * @var \Drupal\entity_workflow\EntityWorkflowInfo
   */
  protected $entityWorkflowInfo;

  /**
   * Constructs a new EntitySchemaSubscriber.
   *
   * @param \Drupal\Core\Entity\EntityDefinitionUpdateManagerInterface $entityDefinitionUpdateManager
   *   Definition update manager.
   * @param \Drupal\Core\Entity\EntityLastInstalledSchemaRepositoryInterface $entityLastInstalledSchemaRepository
   *   Last definitions.
   * @param \Drupal\entity_workflow\EntityWorkflowInfo $entity_workflow_info
   *   The entity workflow information service.
   */
  public function __construct(EntityDefinitionUpdateManagerInterface $entityDefinitionUpdateManager, EntityLastInstalledSchemaRepositoryInterface $entityLastInstalledSchemaRepository, EntityWorkflowInfo $entity_workflow_info) {
    $this->entityDefinitionUpdateManager = $entityDefinitionUpdateManager;
    $this->entityLastInstalledSchemaRepository = $entityLastInstalledSchemaRepository;
    $this->entityWorkflowInfo = $entity_workflow_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return static::getEntityTypeEvents();
  }

  /**
   * {@inheritdoc}
   */
  public function onEntityTypeCreate(EntityTypeInterface $entity_type) {
    // Clear the list of supported entity types so the workflow state field can
    // be added to the new entity type if needed.
    drupal_static_reset('EntityWorkflowContent::defaultConfiguration');
    \Drupal::service('entity_type.bundle.info')->clearCachedBundles();

    // If the entity type is supported by an entity workflow, add the state
    // field.
    if ($this->entityWorkflowInfo->isEntityTypeSupported($entity_type->id())) {
      $this->addWorkflowStateField($entity_type);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onFieldableEntityTypeCreate(EntityTypeInterface $entity_type, array $field_storage_definitions) {
    $this->onEntityTypeCreate($entity_type);
  }

  /**
   * Adds the state field for every workflow that supports this entity type.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type that has been installed or updated.
   */
  protected function addWorkflowStateField(EntityTypeInterface $entity_type) {
    $workflows = $this->entityWorkflowInfo->getWorkflowsInfoForEntityType($entity_type->id());
    foreach ($workflows as $workflow_id => $workflow_label) {
      $field_name = entity_workflow_get_field_name($workflow_id);
      $field_storage_definition = $this->getWorkflowFieldDefinition($workflow_label);

      $this->entityDefinitionUpdateManager->installFieldStorageDefinition($field_name, $entity_type->id(), 'entity_workflow', $field_storage_definition);
    }
  }

  /**
   * Gets the base field definition for the state field of an entity workflow.
   *
   * @param string $workflow_label
   *   The label of the workflow.
   *
   * @return \Drupal\Core\Field\BaseFieldDefinition
   *   The base field definition.
   */
  protected function getWorkflowFieldDefinition($workflow_label) {
    return BaseFieldDefinition::create('entity_workflow_state')
      ->setLabel(t('Entity workflow: @label', ['@label' => $workflow_label]))
      ->setDescription(t('The workflow state of this entity for the @label workflow.', ['@label' => $workflow_label]))
      ->setTranslatable(TRUE)
      ->setRevisionable(TRUE);
  }

}
