<?php

namespace Drupal\entity_workflow\EventSubscriber;

use Drupal\Component\Utility\Environment;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\entity_workflow\Event\EntityWorkflowEvents;
use Drupal\entity_workflow\Event\InitiateTransitionEvent;
use Drupal\entity_workflow\Event\PostTransitionEvent;
use Drupal\entity_workflow\Event\PreTransitionEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class EntityWorkflowEventSubscriber implements EventSubscriberInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new EntityWorkflowEventSubscriber object.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events = [
      EntityWorkflowEvents::INITIATE_TRANSITION => ['onInitiateTransaction', 0],
      PreTransitionEvent::class => ['preTransition', 0],
      PostTransitionEvent::class => ['postTransition', 0]
    ];
    return $events;
  }

  /**
   * Applies the workflow transition
   *
   * @param \Drupal\entity_workflow\Event\InitiateTransitionEvent $event
   *   The initiate transition event.
   */
  public function onInitiateTransaction(InitiateTransitionEvent $event) {
    $entity = $event->getEntity();

    $workflow_field = entity_workflow_get_field($entity, $event->getWorkflow()->id());
    $workflow_field->applyTransition($event->getTransition(), $event->getContext());
    $entity->_entityWorkflowEnforceNoNewRevision = TRUE;
    $entity->save();

    // Extend the execution time after each entity save, so we give ample time
    // for e.g. cascading operations.
    Environment::setTimeLimit(30);
  }

  /**
   * @param \Drupal\entity_workflow\Event\PreTransitionEvent $event
   */
  public function preTransition(PreTransitionEvent $event) {
    /** @var \Drupal\entity_workflow\WorkflowType\EntityWorkflowTypeInterface $workflow_type */
    $workflow_type = $event->getWorkflow()->getTypePlugin();
    $workflow_type->onPreTransition($event->getTransition(), $event->getWorkflow(), $event->getEntity(), $event->getFieldName(), $event->getContext());
  }

  /**
   * @param \Drupal\entity_workflow\Event\PostTransitionEvent $event
   */
  public function postTransition(PostTransitionEvent $event) {
    /** @var \Drupal\entity_workflow\WorkflowType\EntityWorkflowTypeInterface $workflow_type */
    $workflow_type = $event->getWorkflow()->getTypePlugin();
    $workflow_type->onPostTransition($event->getTransition(), $event->getWorkflow(), $event->getEntity(), $event->getFieldName(), $event->getContext());

    $context = $event->getContext();
    if (isset($context['log'])) {
      $workflow_field = $event->getField();

      $this->entityTypeManager->getStorage('workflow_transition_log')->writeLogEntry(
        $event->getEntity(),
        $event->getWorkflow()->id(),
        $workflow_field->getOriginalStateId(),
        $workflow_field->getStateId(),
        $event->getTransition()->id(),
        $context['log']
      );
    }
  }

}
