<?php

namespace Drupal\entity_workflow\Form;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Utility\Error;
use Drupal\entity_workflow\EntityWorkflowInfo;
use Drupal\entity_workflow\StateTransitionValidatorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class EntityWorkflowSimpleTransitionForm extends ConfirmFormBase {

  /**
   * The entity workflow information service.
   *
   * @var \Drupal\entity_workflow\EntityWorkflowInfo
   */
  protected $entityWorkflowInfo;

  /**
   * The state transition validation service.
   *
   * @var \Drupal\entity_workflow\StateTransitionValidatorInterface
   */
  protected $stateTransitionValidator;

  /**
   * @var \Drupal\Core\Entity\ContentEntityInterface
   */
  protected $entity;

  /**
   * @var \Drupal\workflows\WorkflowInterface
   */
  protected $workflow;

  /**
   * @var \Drupal\entity_workflow\WorkflowType\EntityWorkflowTypeInterface
   */
  protected $workflowPlugin;

  /**
   * @var \Drupal\entity_workflow\EntityWorkflowTransition
   */
  protected $transition;

  /**
   * Creates a EntityWorkflowForm object.
   *
   * @param \Drupal\entity_workflow\EntityWorkflowInfo $entity_workflow_info
   *   The entity workflow information service.
   * @param \Drupal\entity_workflow\StateTransitionValidatorInterface $state_transition_validation
   *   The state transition validation service.
   */
  public function __construct(EntityWorkflowInfo $entity_workflow_info, StateTransitionValidatorInterface $state_transition_validation) {
    $this->entityWorkflowInfo = $entity_workflow_info;
    $this->stateTransitionValidator = $state_transition_validation;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_workflow.info'),
      $container->get('entity_workflow.state_transition_validation')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'entity_workflow_simple_transition_form';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->transition->confirmationMessage() ?? $this->t('Are you sure you want to apply this transition?');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->transition->label();
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return $this->entity->toUrl();
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, RouteMatchInterface $route_match = NULL) {
    $this->entity = $route_match->getParameter($route_match->getParameter('entity_type_id'));
    $this->workflow = $route_match->getParameter('workflow');
    $this->workflowPlugin = $this->workflow->getTypePlugin();
    $this->transition = $this->workflowPlugin->getTransition($route_match->getRawParameter('transition_id'));

    // @todo Clean up when Variants provides a proper API for this.
    if (\Drupal::moduleHandler()->moduleExists('location_variant')) {
      $enabled_entity_types = \Drupal::config('variants.settings')->get('entity_types');
      if (in_array($this->entity->getEntityTypeId(), $enabled_entity_types, TRUE)) {
        $langcode = \Drupal::languageManager()->getCurrentLanguage()->getId();

        $candidates = [];
        \Drupal::service('location_variant.negotiator')->getFallbackCandidates($candidates, [
          'data' => $this->entity,
          'langcode' => $langcode,
        ]);
        $langcode = reset($candidates);

        $this->entity = $this->entity->getTranslation($langcode);
      }
    }

    $form = parent::buildForm($form, $form_state);

    if ($this->transition->hasLog()) {
      $form['log'] = [
        '#type' => 'textarea',
        '#title' => $this->t('Provide a log message'),
        '#required' => $this->transition->isLogRequired(),
      ];
    }
    else {
      $form['log'] = [
        '#type' => 'value',
        '#value' => '',
      ];
    }

    // Changing the workflow state of an entity is allowed in a workspace.
    $form_state->set('workspace_safe', TRUE);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $transition = $this->transition;
    $log = $form_state->getValue('log');

    try {
      if ($this->stateTransitionValidator->isTransitionValid($this->entity, $this->workflow, $transition->id(), $this->currentUser())) {
        entity_workflow_transition($this->entity, $this->workflow->id(), $transition->id(), $log);
        $this->messenger()->addMessage($this->t('The workflow state has been changed.'));
      }
      else {
        $this->messenger()->addError($this->t('The workflow state could not be changed.'));
      }
    }
    catch (\Exception $e) {
      Error::logException($this->logger('entity_workflow'), $e);
      $this->messenger()->addError($this->t('An error occurred while changing the workflow state. See logs for more details.'));
    }

    $form_state->setRedirectUrl($this->getCancelUrl());
  }

  /**
   * Checks access for the entity workflow simple transition form.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function checkAccess(RouteMatchInterface $route_match = NULL) {
    $entity = $route_match->getParameter($route_match->getParameter('entity_type_id'));
    $workflow = $route_match->getParameter('workflow');
    $transition_id = $route_match->getRawParameter('transition_id');

    $is_entity_in_workflow = $this->entityWorkflowInfo->isEntityInWorkflow($entity, $workflow->id());
    $is_transition_valid = $this->stateTransitionValidator->isTransitionValid($entity, $workflow, $transition_id, $this->currentUser());

    return AccessResult::allowedIf($is_entity_in_workflow && $is_transition_valid)
      ->addCacheableDependency($entity)
      ->addCacheContexts(['workspace']);
  }

}
