<?php

namespace Drupal\entity_workflow\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\TypedData\OptionsProviderInterface;
use Drupal\Core\Validation\Plugin\Validation\Constraint\AllowedValuesConstraint;
use Drupal\entity_workflow\EntityWorkflowState;
use Drupal\entity_workflow\EntityWorkflowStateItemInterface;
use Drupal\entity_workflow\Event\PostTransitionEvent;
use Drupal\entity_workflow\Event\PreTransitionEvent;
use Drupal\workflows\Entity\Workflow;
use Drupal\workflows\TransitionInterface;

/**
 * Plugin implementation of the 'entity_workflow_state' field type.
 *
 * @FieldType(
 *   id = "entity_workflow_state",
 *   label = @Translation("Entity Workflow State"),
 *   description = @Translation("Stores the current workflow state of an entity."),
 *   default_widget = "options_select",
 *   default_formatter = "list_default",
 *   list_class = "\Drupal\entity_workflow\EntityWorkflowStateFieldItemList",
 *   cardinality = 1,
 * )
 */
class EntityWorkflowStateItem extends FieldItemBase implements EntityWorkflowStateItemInterface, OptionsProviderInterface {

  /**
   * The original value, used to validate state changes.
   *
   * @var string
   */
  protected $originalValue;

  /**
   * The transition to apply.
   *
   * @var \Drupal\workflows\TransitionInterface
   */
  protected $transitionToApply;

  /**
   * The context for the applied transition.
   *
   * @var array
   */
  protected $transitionContext;

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['value'] = DataDefinition::create('string')
      ->setLabel(t('State'))
      ->setRequired(TRUE);

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'value' => [
          'type' => 'varchar_ascii',
          'length' => 255,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getConstraints() {
    $constraints = parent::getConstraints();
    // Replace the 'AllowedValuesConstraint' constraint with the 'State' one.
    foreach ($constraints as $key => $constraint) {
      if ($constraint instanceof AllowedValuesConstraint) {
        unset($constraints[$key]);
      }
    }
    $manager = $this->getTypedDataManager()->getValidationConstraintManager();
    $constraints[] = $manager->create('EntityWorkflowState', []);

    return $constraints;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings() {
    return [
      'workflow' => '',
      'workflow_callback' => '',
    ] + parent::defaultFieldSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function storageSettingsForm(array &$form, FormStateInterface $form_state, $has_data) {
    $element = [];
    // Allow the workflow to be changed if it's not determined by a callback.
    if (!$this->getSetting('workflow_callback')) {
      $options = [];
      foreach (Workflow::loadMultiple() as $workflow) {
        $options[$workflow->id()] = $workflow->label();
      }

      $element['workflow'] = [
        '#type' => 'select',
        '#title' => $this->t('Workflow'),
        '#options' => $options,
        '#default_value' => $this->getSetting('workflow'),
        '#disabled' => $has_data,
        '#required' => TRUE,
      ];
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    // Note that in this field's case the value will never be empty
    // because of the default returned in applyDefaultValue().
    return $this->value === NULL || $this->value === '';
  }

  /**
   * {@inheritdoc}
   */
  public function applyDefaultValue($notify = TRUE) {
    // @todo Revisit the default value handling.
//    if ($workflow = $this->getWorkflow()) {
//      $initial_state = $workflow->getTypePlugin()->getInitialState();
//      $this->setValue(['value' => $initial_state->id()], $notify);
//    }
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setValue($values, $notify = TRUE) {
    if (empty($this->originalValue)) {
      // If no array is given, then the method received just the state value.
      if (isset($values) && !is_array($values)) {
        $values = ['value' => $values];
      }
      // Track the original field value to allow isValid() to validate changes
      // and to react to transitions.
      $this->originalValue = $values['value'];
    }
    parent::setValue($values, $notify);
  }

  /**
   * {@inheritdoc}
   */
  public function isValid() {
    $allowed_states = $this->getAllowedStates($this->originalValue);
    return isset($allowed_states[$this->value]);
  }

  /**
   * {@inheritdoc}
   */
  public function getPossibleValues(AccountInterface $account = NULL) {
    return array_keys($this->getPossibleOptions($account));
  }

  /**
   * {@inheritdoc}
   */
  public function getPossibleOptions(AccountInterface $account = NULL) {
    $workflow = $this->getWorkflow();
    return array_map([EntityWorkflowState::class, 'labelCallback'], $workflow->getTypePlugin()->getStates());
  }

  /**
   * {@inheritdoc}
   */
  public function getSettableValues(AccountInterface $account = NULL) {
    return array_keys($this->getSettableOptions($account));
  }

  /**
   * {@inheritdoc}
   */
  public function getSettableOptions(AccountInterface $account = NULL) {
    $allowed_states = $this->getAllowedStates($this->value);
    return array_map([EntityWorkflowState::class, 'labelCallback'], $allowed_states);
  }

  /**
   * Gets the next allowed states for the given field value.
   *
   * @param string $value
   *   The field value, representing the state ID.
   *
   * @return \Drupal\workflows\StateInterface[]
   *   The allowed states.
   */
  protected function getAllowedStates($value) {
    $allowed_states = [];
    foreach ($this->getAllowedTransitions($value) as $transition) {
      $state = $transition->to();
      $allowed_states[$state->id()] = $state;
    }

    return $allowed_states;
  }

  /**
   * {@inheritdoc}
   */
  public function getWorkflow() {
    if ($callback = $this->getSetting('workflow_callback')) {
      $workflow_id = call_user_func($callback, $this->getFieldDefinition());
    }
    else {
      $workflow_id = $this->getSetting('workflow');
    }

    return Workflow::load($workflow_id);
  }

  /**
   * {@inheritdoc}
   */
  public function getOriginalStateId() {
    return $this->originalValue;
  }

  /**
   * {@inheritdoc}
   */
  public function getStateId() {
    return $this->value ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getStateLabel() {
    if (!$this->value) {
      return $this->t('N/A');
    }

    return $this->getWorkflow()->getTypePlugin()->getState($this->value)->label();
  }

  /**
   * Gets the allowed transitions for the given state ID.
   *
   * @param string $value
   *   A state ID.
   *
   * @return \Drupal\workflows\TransitionInterface[]
   *   The allowed transitions.
   */
  protected function getAllowedTransitions($value) {
    /** @var \Drupal\entity_workflow\StateTransitionValidatorInterface $state_transition_validator */
    $state_transition_validator = \Drupal::service('entity_workflow.state_transition_validation');
    $workflow = $this->getWorkflow();

    $allowed_transitions = [];
    foreach ($workflow->getTypePlugin()->getTransitionsForState($value) as $transition_id => $transition) {
      if ($state_transition_validator->isTransitionValid($this->getEntity(), $workflow, $transition_id, \Drupal::currentUser())) {
        $allowed_transitions[$transition_id] = $transition;
      }
    }

    return $allowed_transitions;
  }

  /**
   * {@inheritdoc}
   */
  public function getTransitions() {
    return $this->getAllowedTransitions($this->value);
  }

  /**
   * {@inheritdoc}
   */
  public function isTransitionAllowed($transition_id) {
    $allowed_transitions = $this->getAllowedTransitions($this->value);
    return isset($allowed_transitions[$transition_id]) || $transition_id === SYSTEM_TRANSITION;
  }

  /**
   * {@inheritdoc}
   */
  public function applyTransition(TransitionInterface $transition, $context = []) {
//    if (!$this->isTransitionAllowed($transition->id())) {
//      throw new \InvalidArgumentException(sprintf('The transition "%s" is not allowed. (Current state: "%s".)', $transition->id(), $this->getStateId()));
//    }
    // Store the transition to apply, to ensure we're applying the requested
    // transition instead of guessing based on the original state.
    $this->transitionToApply = $transition;
    $this->setValue(['value' => $transition->to()->id()]);

    $this->transitionContext = $context;

    // Default transitions must not set 'originalValue' on the field item,
    // otherwise the transition events won't be invoked.
    $default_transition = $context['default_transition'] ?? FALSE;
    if ($default_transition) {
      $this->originalValue = NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function applyTransitionById($transition_id, $context = []) {
    $transition = $this->getWorkflow()->getTypePlugin()->getTransition($transition_id);
    $this->applyTransition($transition, $context);
  }

  /**
   * {@inheritdoc}
   */
  public function preSave() {
    if ($this->value != $this->originalValue) {
      $this->dispatchTransitionEvent('pre_transition');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function postSave($update) {
    if ($this->value != $this->originalValue) {
      $this->dispatchTransitionEvent('post_transition');
    }
    $this->originalValue = $this->value;
    // Reset the transition to apply to ensure the next entity save doesn't
    // trigger the same transition by mistake.
    $this->transitionToApply = NULL;
    $this->transitionContext = NULL;
  }

  /**
   * Invokes a transition event for the given phase.
   *
   * @param string $phase
   *   The phase: pre_transition OR post_transition.
   */
  protected function dispatchTransitionEvent($phase) {
    $workflow = $this->getWorkflow();
    if (isset($this->transitionToApply) || $workflow->getTypePlugin()->hasTransitionFromStateToState($this->originalValue, $this->value)) {
      $transition = $this->transitionToApply ?? $workflow->getTypePlugin()->getTransitionFromStateToState($this->originalValue, $this->value);

      // Dispatch the event.
      $field_name = $this->getFieldDefinition()->getName();
      $context = $this->transitionContext ?? [];
      $event_dispatcher = \Drupal::getContainer()->get('event_dispatcher');
      switch ($phase) {
        case 'pre_transition':
          $event = new PreTransitionEvent($transition, $workflow, $this->getEntity(), $field_name, $context);
          break;
        case 'post_transition':
          $event = new PostTransitionEvent($transition, $workflow, $this->getEntity(), $field_name, $context);
          break;
      }
      $event_dispatcher->dispatch($event);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function calculateDependencies(FieldDefinitionInterface $field_definition) {
    $dependencies = [];

    if ($callback = $field_definition->getSetting('workflow_callback')) {
      $workflow_id = call_user_func($callback, $field_definition);
    }
    else {
      $workflow_id = $field_definition->getSetting('workflow');
    }
    if ($workflow_id && ($workflow = Workflow::load($workflow_id))) {
      $dependencies['config'][] = $workflow->getConfigDependencyName();
    }

    return $dependencies;
  }

}
