<?php

namespace Drupal\entity_workflow\Routing;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\RouteSubscriberBase;
use Drupal\Core\Routing\RoutingEvents;
use Drupal\entity_workflow\EntityWorkflowInfo;
use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;

/**
 * Subscriber for Entity Workflow routes.
 */
class RouteSubscriber extends RouteSubscriberBase {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity workflow information service.
   *
   * @var \Drupal\entity_workflow\EntityWorkflowInfo
   */
  protected $entityWorkflowInfo;

  /**
   * Constructs a RouteSubscriber object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\entity_workflow\EntityWorkflowInfo $entity_workflow_info
   *   The entity workflow info service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, EntityWorkflowInfo $entity_workflow_info) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityWorkflowInfo = $entity_workflow_info;
  }

  /**
   * {@inheritdoc}
   */
  protected function alterRoutes(RouteCollection $collection) {
    foreach ($this->entityTypeManager->getDefinitions() as $entity_type_id => $entity_type) {
      if ($this->entityWorkflowInfo->isEntityTypeSupported($entity_type_id)) {
        $base_path = NULL;
        if ($entity_type->hasLinkTemplate('canonical')) {
          $base_path = $entity_type->getLinkTemplate('canonical');
        }
        elseif ($entity_type->hasLinkTemplate('edit-form')) {
          $base_path = $entity_type->getLinkTemplate('edit-form');
        }

        if (!$base_path) {
          continue;
        }

        $parameters = [
          $entity_type_id => [
            'type' => "entity:$entity_type_id",
          ],
          'workflow' => [
            'type' => 'entity:workflow',
          ],
        ];

        // Add a route for the workflow form.
        $route = new Route($base_path . '/workflow/{workflow}');
        $route
          ->addDefaults([
            '_controller' => '\Drupal\entity_workflow\Controller\EntityWorkflowController::workflow',
            'entity_type_id' => $entity_type_id,
          ])
          ->setRequirements([
            '_custom_access' => '\Drupal\entity_workflow\Controller\EntityWorkflowController::checkWorkflowAccess'
          ])
          ->setOption('parameters', $parameters)
          ->setOption('_admin_route', TRUE);
        $collection->add("entity.$entity_type_id.workflow", $route);

        // Add a route for the bulk workflow form. Workspaces don't need bulk
        // workflows for themselves.
        if ($entity_type_id !== 'workspace') {
          $route = new Route($base_path . '/bulk-workflow/{workflow}');
          $route
            ->addDefaults([
              '_controller' => '\Drupal\entity_workflow\Controller\EntityWorkflowController::bulkWorkflow',
              'entity_type_id' => $entity_type_id,
            ])
            ->setRequirements([
              '_custom_access' => '\Drupal\entity_workflow\Controller\EntityWorkflowController::checkBulkWorkflowAccess'
            ])
            ->setOption('parameters', $parameters)
            ->setOption('_admin_route', TRUE);
          $collection->add("entity.$entity_type_id.bulk_workflow", $route);
        }

        // Add a route for the simple transition form.
        $route = new Route($base_path . '/workflow/{workflow}/{transition_id}');
        $route
          ->addDefaults([
            '_form' => '\Drupal\entity_workflow\Form\EntityWorkflowSimpleTransitionForm',
            'entity_type_id' => $entity_type_id,
            'transition_id' => '',
          ])
          ->setRequirements([
            '_custom_access' => '\Drupal\entity_workflow\Form\EntityWorkflowSimpleTransitionForm::checkAccess'
          ])
          ->setOption('parameters', $parameters)
          ->setOption('_admin_route', TRUE);
        $collection->add("entity.$entity_type_id.workflow_transition", $route);
      }
    }

    // Add a route to workspaces for bulk operations.
    if ($entity_type = $this->entityTypeManager->getDefinition('workspace', FALSE)) {
      $route = new Route($entity_type->getLinkTemplate('canonical') . '/bulk-workflow/{workflow}');
      $route
        ->addDefaults([
          '_controller' => '\Drupal\entity_workflow\Controller\EntityWorkflowController::bulkWorkflowWorkspace',
        ])
        ->setRequirements([
          '_custom_access' => '\Drupal\entity_workflow\Controller\EntityWorkflowController::checkBulkWorkflowWorkspaceAccess'
        ])
        ->setOption('parameters', [
          'workspace' => [
            'type' => "entity:workspace",
          ],
          'workflow' => [
            'type' => 'entity:workflow',
          ],
        ])
        ->setOption('_admin_route', TRUE);
      $collection->add("entity.workspace.bulk_workflow", $route);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events = parent::getSubscribedEvents();
    $events[RoutingEvents::ALTER] = ['onAlterRoutes', -160];
    return $events;
  }

}
