<?php

namespace Drupal\entity_workflow\WorkflowType;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\workflows\TransitionInterface;
use Drupal\workflows\WorkflowInterface;
use Drupal\workflows\WorkflowTypeInterface;

/**
 * Interface for EntityWorkflowTypeBase WorkflowType plugins.
 */
interface EntityWorkflowTypeInterface extends WorkflowTypeInterface {

  /**
   * Gets the entity types the workflow is applied to.
   *
   * @return string[]
   *   The entity types the workflow is applied to.
   */
  public function getEntityTypes();

  /**
   * Gets any bundles the workflow is applied to for the given entity type.
   *
   * @param string $entity_type_id
   *   The entity type ID to get the bundles for.
   *
   * @return string[]
   *   The bundles of the entity type the workflow is applied to or an empty
   *   array if the entity type is not applied to the workflow.
   */
  public function getBundlesForEntityType($entity_type_id);

  /**
   * Checks if the workflow applies to the supplied entity type and bundle.
   *
   * @param string $entity_type_id
   *   The entity type ID to check.
   * @param string $bundle_id
   *   The bundle ID to check.
   *
   * @return bool
   *   TRUE if the workflow applies to the supplied entity type ID and bundle
   *   ID. FALSE if not.
   */
  public function appliesToEntityTypeAndBundle($entity_type_id, $bundle_id);

  /**
   * Gets the default state of this workflow.
   *
   * @return string|null
   *   The default state ID, if set.
   */
  public function getDefaultState();

  /**
   * Gets the default state callback of this workflow.
   *
   * @return string|null
   *   The default state callback, if set.
   */
  public function getDefaultStateCallback();

  /**
   * Gets the default transition of this workflow.
   *
   * @return string|null
   *   The default transition ID, if set.
   */
  public function getDefaultTransition();

  /**
   * Gets the default transition log of this workflow.
   *
   * @return string|null
   *   The default transition log, if set.
   */
  public function getDefaultTransitionLog();

  /**
   * Returns an array of permissions for this workflow type.
   *
   * @return array
   *   The dynamic permissions.
   */
  public function getPermissions();

  /**
   * {@inheritdoc}
   *
   * @return \Drupal\entity_workflow\EntityWorkflowState[]
   */
  public function getStates($state_ids = NULL);

  /**
   * {@inheritdoc}
   *
   * @return \Drupal\entity_workflow\EntityWorkflowState
   */
  public function getState($state_id);

  /**
  * {@inheritdoc}
   *
   * @return \Drupal\entity_workflow\EntityWorkflowTransition[]
   */
  public function getTransitions(array $transition_ids = NULL);

  /**
  * {@inheritdoc}
   *
   * @return \Drupal\entity_workflow\EntityWorkflowTransition
   */
  public function getTransition($transition_id);

  /**
   * Gets a system transition object for the provided 'to' state ID.
   *
   * @param string $to
   *   The 'to' state ID.
   *
   * @return \Drupal\entity_workflow\EntityWorkflowTransition
   *   The transition.
   */
  public function getSystemTransition($to);

  /**
   * Operates before a workflow transition is applied.
   *
   * @param \Drupal\workflows\TransitionInterface $transition
   *   The transition applied.
   * @param \Drupal\workflows\WorkflowInterface $workflow
   *   The workflow of the transition.
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity on which the transition was applied.
   * @param string $field_name
   *   The field name which hold the workflow state.
   * @param array $context
   *   The transition context.
   */
  public function onPreTransition(TransitionInterface $transition, WorkflowInterface $workflow, ContentEntityInterface $entity, $field_name, array $context = []);

  /**
   * Operates after a workflow transition is applied.
   *
   * @param \Drupal\workflows\TransitionInterface $transition
   *   The transition applied.
   * @param \Drupal\workflows\WorkflowInterface $workflow
   *   The workflow of the transition.
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity on which the transition was applied.
   * @param string $field_name
   *   The field name which hold the workflow state.
   * @param array $context
   *   The transition context.
   */
  public function onPostTransition(TransitionInterface $transition, WorkflowInterface $workflow, ContentEntityInterface $entity, $field_name, array $context = []);

}
